<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 12/5/19
 * Time: 11:09 AM
 */

namespace IMATHUZH\Qfq\Core\Form;

//use IMATHUZH\Qfq\Core\Database\Database;
//use IMATHUZH\Qfq\Core\Helper\HelperFile;
//use IMATHUZH\Qfq\Core\Helper\HelperFormElement;
//use IMATHUZH\Qfq\Core\Helper\KeyValueStringParser;
//use IMATHUZH\Qfq\Core\Helper\Ldap;
//use IMATHUZH\Qfq\Core\Helper\Logger;
//use IMATHUZH\Qfq\Core\Helper\OnArray;
//use IMATHUZH\Qfq\Core\Helper\Sanitize;
use IMATHUZH\Qfq\Core\Helper\HelperFormElement;
use IMATHUZH\Qfq\Core\Helper\Support;
use IMATHUZH\Qfq\Core\Store\Store;

//use IMATHUZH\Qfq\Core\Report\Link;
//use IMATHUZH\Qfq\Core\Report\Report;
//use IMATHUZH\Qfq\Core\Store\Sip;


/**
 * Class Checkbox
 * @package IMATHUZH\Qfq\Core\Form
 */
class Checkbox {

    /**
     * @var Store
     */
    private $store = null;

    /**
     * Checkbox constructor.
     * @throws \CodeException
     * @throws \UserFormException
     * @throws \UserReportException
     */
    public function __construct() {

        $this->store = Store::getInstance();
    }

    /**
     * Builds HTML 'checkbox' element.
     *
     * There are 6 different modes:
     *
     * * buildCheckboxSingle
     *
     *   * SB) constructCheckboxSingleBootstrap
     *   * SP) constructCheckboxSinglePlain
     *
     * * buildCheckboxMulti
     *
     *   * constructCheckboxMultiButton
     *     * MBH) Horizontal
     *     * MBV) Vertical
     *
     *   * constructCheckboxMultiPlain
     *     * MPH) Horizontal
     *     * MPV) Vertical
     *
     * @param array $formElement
     * @param string $htmlFormElementName
     * @param string $value
     * @param array $json
     * @param string $mode FORM_LOAD | FORM_UPDATE | FORM_SAVE*
     *
     * @return string
     * @throws \CodeException
     * @throws \UserFormException
     * @throws \UserReportException
     */
    public function build(array $formElement, $htmlFormElementName, $value, array &$json, $mode) {

        $itemKey = array();
        $itemValue = array();

        // Fill $itemKey & $itemValue
        HelperFormElement::getKeyValueListFromSqlEnumSpec($formElement, $itemKey, $itemValue);

        // Get fallback, if 'checkBoxMode' is not defined:
        if (!isset($formElement['checkBoxMode'])) {
            // This fallback is problematic if 'set' or 'enum' has 2 : defaults to single but maybe multi is meant.
            $formElement['checkBoxMode'] = (count($itemKey) > 2) ? 'multi' : 'single';
        }

        if ($formElement['checkBoxMode'] === 'multi') {

        } else {
            // Fill meaningful defaults to parameter: checked|unchecked  (CHECKBOX_VALUE_CHECKED|CHECKBOX_VALUE_UNCHECKED)
            $this->prepareCheckboxCheckedUncheckedValue($itemKey, $formElement);
        }

        $attributeBase = HelperFormElement::getAttributeFeMode($formElement[FE_MODE]);
        $attributeBase .= Support::doAttribute('type', $formElement[FE_TYPE]);
        $attributeBase .= HelperFormElement::getAttributeList($formElement, [F_FE_DATA_REQUIRED_ERROR]);

        switch ($formElement['checkBoxMode']) {
            case 'single':
                $html = $this->buildCheckboxSingle($formElement, $htmlFormElementName, $attributeBase, $value, $json);
                break;
            case 'multi';
                $html = $this->buildCheckboxMulti($formElement, $htmlFormElementName, $attributeBase, $value, $itemKey, $itemValue, $json);
                break;
            default:
                throw new \UserFormException('checkBoxMode: \'' . $formElement['checkBoxMode'] . '\' is unknown.', ERROR_CHECKBOXMODE_UNKNOWN);
        }

        $formElement = HelperFormElement::prepareExtraButton($formElement, false);

        return $html . $formElement[FE_TMP_EXTRA_BUTTON_HTML] . HelperFormElement::getHelpBlock() . $formElement[FE_INPUT_EXTRA_BUTTON_INFO];
    }

    /**
     * For CheckBox's with only one checkbox: if parameter.checked|unchecked is not defined, take defaults:
     *
     *    checked: first Element in $itemKey
     *  unchecked: ''
     *
     * @param array $itemKey
     * @param array $formElement
     *
     * @throws \CodeException
     * @throws \UserFormException
     */
    private function prepareCheckboxCheckedUncheckedValue(array $itemKey, array &$formElement) {

        if (!isset($formElement[FE_CHECKBOX_CHECKED])) {
            if (isset($itemKey[0])) {
                // First element in $itemKey list
                $formElement[FE_CHECKBOX_CHECKED] = $itemKey[0];
            } else {
                // Take column default value
                $formElement[FE_CHECKBOX_CHECKED] = $this->store->getVar($formElement[FE_NAME], STORE_TABLE_DEFAULT);
            }
        }

        // unchecked
        if (!isset($formElement[FE_CHECKBOX_UNCHECKED])) {
            if (isset($itemKey[1])) {
                $formElement[FE_CHECKBOX_UNCHECKED] = ($itemKey[0] === $formElement[FE_CHECKBOX_CHECKED]) ? $itemKey[1] : $itemKey[0];
            } else {
                $formElement[FE_CHECKBOX_UNCHECKED] = '';
            }
        }

        if ($formElement[FE_CHECKBOX_CHECKED] === $formElement[FE_CHECKBOX_UNCHECKED]) {
            throw new \UserFormException('FormElement: type=checkbox - checked and unchecked can\'t be the same: ' . $formElement[FE_CHECKBOX_CHECKED], ERROR_CHECKBOX_EQUAL);
        }
    }

    /**
     * Build a Multi Checkbox either in HTML plain layout or with Bootstrap Button class.
     *
     * @param array $formElement
     * @param string $htmlFormElementName
     * @param $attributeBase
     * @param string $value
     * @param array $itemKey
     * @param array $itemValue
     * @param array $json
     * @return string
     * @throws \CodeException
     * @throws \UserFormException
     */
    public function buildCheckboxMulti(array $formElement, $htmlFormElementName, $attributeBase, $value, array $itemKey, array $itemValue, array &$json) {

        if (isset($formElement[FE_BUTTON_CLASS])) {

            if ($formElement[FE_BUTTON_CLASS] == '') {
                $formElement[FE_BUTTON_CLASS] = 'btn-default';
            }
            $formElement[FE_BUTTON_CLASS] = 'btn ' . $formElement[FE_BUTTON_CLASS];

            if ($formElement[FE_MODE] == FE_MODE_READONLY) {
                $formElement[FE_BUTTON_CLASS] .= ' disabled';
            }

            return $this->constructCheckboxMultiButton($formElement, $htmlFormElementName, $attributeBase, $value, $itemKey, $itemValue, $json);
        } else {
            return $this->constructCheckboxMultiPlain($formElement, $htmlFormElementName, $attributeBase, $value, $itemKey, $itemValue, $json);
        }
    }

    /**
     * Build as many Checkboxes as items.
     *
     * Layout: The Bootstrap Layout needs very special setup, the checkboxes are wrapped differently with <div
     * class=checkbox> depending of if they aligned horizontal or vertical.
     * @param array $formElement
     * @param string $htmlFormElementName
     * @param string $attributeBase
     * @param string $value
     * @param array $itemKey
     * @param array $itemValue
     * @param array $json
     *
     * @return string
     * @throws \CodeException
     * @throws \UserFormException
     */
    private function constructCheckboxMultiButton(array $formElement, $htmlFormElementName, $attributeBase, $value, array $itemKey, array $itemValue, array &$json) {
        $json = array();

        // Defines which of the checkboxes will be checked.
        $values = explode(',', $value);

        $attributeBase .= Support::doAttribute('data-load', ($formElement[FE_DYNAMIC_UPDATE] === 'yes') ? 'data-load' : '');
        if ($formElement[FE_MODE] == FE_MODE_READONLY) {
            $attributeBase .= Support::doAttribute('disabled', 'disabled');
        }

        $html = '';

        // Used in getFormElementForJson() for dynamic update.
        $labelBaseClass = $formElement[FE_BUTTON_CLASS];

        for ($ii = 0, $jj = 1; $ii < count($itemKey); $ii++, $jj++) {

            $attribute = $attributeBase;
            $jsonValue = false;
            $classActive = '';

            $htmlFormElementNameUniq = HelperFormElement::prependFormElementNameCheckBoxMulti($htmlFormElementName, $ii, true);
            $attribute .= Support::doAttribute('id', HelperFormElement::getCheckboxRadioOptionId($formElement[FE_HTML_ID], $ii));
            $attribute .= Support::doAttribute('name', $htmlFormElementNameUniq);
            $attribute .= Support::doAttribute(ATTRIBUTE_DATA_REFERENCE, $formElement[FE_DATA_REFERENCE] . '-' . $ii);

            if (isset($formElement[FE_AUTOFOCUS])) {
                $attribute .= Support::doAttribute('autofocus', $formElement[FE_AUTOFOCUS]);
                unset ($formElement[FE_AUTOFOCUS]);
            }

            $attribute .= Support::doAttribute('value', $itemKey[$ii], false);

            // Check if the given key is found in field.
            if (false !== array_search($itemKey[$ii], $values)) {
                $attribute .= Support::doAttribute('checked', 'checked');
                $jsonValue = true;
                $classActive = ' active';
            }

            // '&nbsp;' - This is necessary to correctly align an empty input.
            $valueShow = ($itemValue[$ii] === '') ? '&nbsp;' : $itemValue[$ii];

            $htmlElement = '<input ' . $attribute . '>' . $valueShow;

            $labelAttribute = Support::doAttribute('title', $formElement[FE_TOOLTIP]);
            $labelAttribute .= Support::doAttribute('class', $formElement[FE_BUTTON_CLASS] . $classActive);
            $labelAttribute .= Support::doAttribute('id', HelperFormElement::getCheckboxRadioOptionId($formElement[FE_HTML_ID], $ii, HTML_ID_EXTENSION_LABEL));
            $html .= Support::wrapTag("<label $labelAttribute>", $htmlElement, true);

            $formElement[FE_TMP_CLASS_OPTION] = $labelBaseClass . $classActive;

            $json[] = $this->getFormElementForJsonCheckBox($htmlFormElementNameUniq, $jsonValue, $formElement, $ii);
        }

        $html = Support::wrapTag('<div class="btn-group" data-toggle="buttons">', $html);

        return $html;
    }

    /**
     * Build as many Checkboxes as items.
     *
     * Layout: The Bootstrap Layout needs very special setup, the checkboxes are wrapped differently with <div
     * class=checkbox> depending of if they aligned horizontal or vertical.
     *
     *  <div class="col-md-6" required="required">
     *    <label class="checkbox-inline" id="1039-21332-1-0-0-l">
     *      <input checked="checked" id="1039-21332-1-0-0" name="groups-1[]" required="required" type="checkbox" value="1">
     *        label1
     *        <span aria="hidden" class="checkmark"></span>
     *      </label>
     *
     *    <label class="checkbox-inline" id="1039-21332-1-0-2-l">
     *      <input id="1039-21332-1-2-0" name="groups-1[]"
     *        required="required" type="checkbox" value="2">
     *        label2
     *        <span aria="hidden" class="checkmark"></span>
     *    </label>
     *  </div>
     *
     * @param array $formElement
     * @param string $htmlFormElementName
     * @param string $attributeBase
     * @param string $value
     * @param array $itemKey
     * @param array $itemValue
     * @param array $json
     *
     * @return string
     * @throws \CodeException
     * @throws \UserFormException
     */
    private function constructCheckboxMultiPlain(array $formElement, $htmlFormElementName, $attributeBase, $value, array $itemKey, array $itemValue, array &$json) {
        $json = array();

        // Defines which of the checkboxes will be checked.
        $values = explode(',', $value);

        $attributeBase .= Support::doAttribute('data-load', ($formElement[FE_DYNAMIC_UPDATE] === 'yes') ? 'data-load' : '');

        $attributeBaseLabel = Support::doAttribute('style', 'min-width: ' . $formElement[F_FE_MIN_WIDTH] . 'px;');

        $html = '';

        $orientation = ($formElement[FE_MAX_LENGTH] > 1) ? ALIGN_HORIZONTAL : ALIGN_VERTICAL;
        $checkboxClass = ($orientation === ALIGN_HORIZONTAL) ? 'checkbox-inline' : 'checkbox';
        if ($formElement[FE_MODE] == FE_MODE_READONLY) {
            $checkboxClass .= ' qfq-disabled'; // necessary for own style checkboxes to display them 'disabled'
        }

        // Used in getFormElementForJson() for dynamic update.
        $formElement[FE_TMP_CLASS_OPTION] = $checkboxClass;

        $br = '';

        for ($ii = 0, $jj = 1; $ii < count($itemKey); $ii++, $jj++) {
            $jsonValue = false;
            $attribute = $attributeBase;
            $htmlFormElementNameUniq = HelperFormElement::prependFormElementNameCheckBoxMulti($htmlFormElementName, $ii, true);
            $attribute .= Support::doAttribute('id', HelperFormElement::getCheckboxRadioOptionId($formElement[FE_HTML_ID], $ii));
            $attribute .= Support::doAttribute('name', $htmlFormElementNameUniq);
            $attribute .= Support::doAttribute(ATTRIBUTE_DATA_REFERENCE, $formElement[FE_DATA_REFERENCE] . '-' . $ii);

            if (isset($formElement[FE_AUTOFOCUS])) {
                $attribute .= Support::doAttribute('autofocus', $formElement[FE_AUTOFOCUS]);
                unset ($formElement[FE_AUTOFOCUS]);
            }

            $attribute .= Support::doAttribute('value', $itemKey[$ii], false);

            // Check if the given key is found in field.
            if (false !== array_search($itemKey[$ii], $values)) {
                $attribute .= Support::doAttribute('checked', 'checked');
                $jsonValue = true;
            }

            // '&nbsp;' - This is necessary to correctly align an empty input.
            $valueShow = ($itemValue[$ii] === '') ? '&nbsp;' : $itemValue[$ii];

            $htmlElement = '<input ' . $attribute . '>' . $valueShow;

            $checkboxLabelId = HelperFormElement::getCheckboxRadioOptionId($formElement[FE_HTML_ID], $ii, HTML_ID_EXTENSION_LABEL);
            $htmlElement = Support::wrapTag("<label class=\"$checkboxClass\" $attributeBaseLabel id=\"$checkboxLabelId\">", $htmlElement, true);

            // control orientation
            if ($formElement[FE_MAX_LENGTH] > 1) {

                if ($jj == $formElement[FE_MAX_LENGTH]) {
                    $jj = 0;
                    $br = '<br>';
                } else {
                    $br = '';
                }
            }

            $html .= $htmlElement . $br;
            $json[] = $this->getFormElementForJsonCheckbox($htmlFormElementNameUniq, $jsonValue, $formElement, $ii);

        }

        return $html;
    }

    /**
     * Helper Function to collect hidden checkbox elements.
     *
     * @param array $formElement
     * @param string $htmlFormElementName
     * @param string $htmlHidden
     * @throws \CodeException
     * @throws \UserFormException
     */
    private function fillStoreAdditionalFormElementsCheckboxHidden(array $formElement, $htmlFormElementName, $htmlHidden) {

        if (isset($formElement[NAME_TG_COPIES]) && $formElement[NAME_TG_COPIES] > 0) {
            for ($ii = 1; $ii <= $formElement[NAME_TG_COPIES]; $ii++) {
                $key = str_replace('%d', $ii, $htmlFormElementName);
                $value = str_replace('%d', $ii, $htmlHidden);
                $this->store->setVar($key, $value, STORE_ADDITIONAL_FORM_ELEMENTS, true);
            }
        } else {
            $this->store->setVar($htmlFormElementName, $htmlHidden, STORE_ADDITIONAL_FORM_ELEMENTS, false);
        }
    }


    /**
     * Build a Checkbox based on two values. Either in HTML plain layout or with Bootstrap Button class.
     *
     * @param array $formElement
     * @param string $htmlFormElementName
     * @param string $attribute
     * @param string $value
     * @param array $json
     * @param string $mode FORM_LOAD | FORM_UPDATE | FORM_SAVE
     *
     * @return string
     * @throws \CodeException
     * @throws \UserFormException
     */
    public function buildCheckboxSingle(array $formElement, $htmlFormElementName, $attribute, $value, array &$json, $mode = FORM_LOAD) {

        // Bootstrap
        $attribute .= Support::doAttribute('id', $formElement[FE_HTML_ID] . '-0');
        $attribute .= Support::doAttribute('name', $htmlFormElementName);
        $attribute .= Support::doAttribute('value', $formElement[FE_CHECKBOX_CHECKED], false);
        $attribute .= Support::doAttribute('data-load', ($formElement[FE_DYNAMIC_UPDATE] === 'yes') ? 'data-load' : '');
        $attribute .= Support::doAttribute(ATTRIBUTE_DATA_REFERENCE, $formElement[FE_DATA_REFERENCE]);

        $classActive = '';
        $valueJson = false;

        if ($formElement[FE_CHECKBOX_CHECKED] === $value) {
            $attribute .= Support::doAttribute('checked', 'checked');
            $valueJson = $value;
        }

        //$attribute .= HelperFormElement::getAttributeList($formElement, [FE_AUTOFOCUS, F_FE_DATA_PATTERN_ERROR, F_FE_DATA_REQUIRED_ERROR, F_FE_DATA_MATCH_ERROR, F_FE_DATA_ERROR]);
        $attribute .= HelperFormElement::getAttributeList($formElement, [FE_AUTOFOCUS, F_FE_DATA_REQUIRED_ERROR]);

        if (isset($formElement[FE_BUTTON_CLASS])) {
            // Bootstrap
            return $this->constructCheckboxSingleBootstrap($formElement, $htmlFormElementName, $attribute, $value, $json, $valueJson);
        } else {
            // Plain
            return $this->constructCheckboxSinglePlain($formElement, $htmlFormElementName, $attribute, $value, $json, $valueJson);
        }
    }

    /**
     * Build a Checkbox based on two values with Bootstrap Button class.
     *
     * <div class="btn-group" data-toggle="buttons">
     *    <input type="hidden" name="$htmlFormElementName" value="$valueUnChecked">
     *    <label class="btn btn-primary active">
     *       <input type="checkbox" autocomplete="off" name="$htmlFormElementName" value="$valueChecked"checked>
     *       Checkbox 1 (pre-checked)
     *    </label>
     * </div>
     *
     * @param array $formElement
     * @param string $htmlFormElementName
     * @param string $attribute
     * @param string $value
     * @param array $json
     *
     * @return string
     * @throws \CodeException
     * @throws \UserFormException
     */
    public function constructCheckboxSingleBootstrap(array $formElement, $htmlFormElementName, $attribute, $value, array &$json, $valueJson) {
        $html = '';

        if ($formElement[FE_BUTTON_CLASS] == '') {
            $formElement[FE_BUTTON_CLASS] = 'btn-default';
        }
        $formElement[FE_BUTTON_CLASS] = 'btn ' . $formElement[FE_BUTTON_CLASS];

        if ($formElement[FE_MODE] == FE_MODE_READONLY) {
            $formElement[FE_BUTTON_CLASS] .= ' disabled';
        }

        $classActive = '';
        if ($formElement[FE_CHECKBOX_CHECKED] === $value) {
            $formElement[FE_BUTTON_CLASS] .= ' active';
        }

        $htmlHidden = HelperFormElement::buildNativeHidden($htmlFormElementName, $formElement[FE_CHECKBOX_UNCHECKED]);
        $this->store->setVar($htmlFormElementName, $htmlHidden, STORE_ADDITIONAL_FORM_ELEMENTS, false);
        $html = '';

        $htmlElement = '<input ' . $attribute . '>';
        if (isset($formElement['label2'])) {
            $htmlElement .= $formElement['label2'];
        } else {
            $htmlElement .= $formElement['checked'];
        }

        $labelAttribute = Support::doAttribute('title', $formElement[FE_TOOLTIP]);
        $labelAttribute .= Support::doAttribute('class', $formElement[FE_BUTTON_CLASS]);
        $labelAttribute .= Support::doAttribute('id', HelperFormElement::getCheckboxRadioOptionId($formElement[FE_HTML_ID], 0, HTML_ID_EXTENSION_LABEL));
        $html .= Support::wrapTag("<label $labelAttribute>", $htmlElement, true);
        $html = Support::wrapTag('<div class="btn-group" data-toggle="buttons">', $html);

        $formElement[FE_TMP_CLASS_OPTION] = $formElement[FE_BUTTON_CLASS];
        $json = $this->getFormElementForJsonCheckbox($htmlFormElementName, $valueJson, $formElement);

        return $html;
    }

    /**
     * Create an array with standard elements for 'mode' (hidden, disabled, required, readonly)
     * and add 'form-element', 'value'.
     * 'Generic Element Update': add via API_ELEMENT_UPDATE 'label' and 'note'.
     * All collected data as array - will be later converted to JSON.
     *
     * @param string $htmlFormElementName
     * @param string|array $value
     * @param array $formElement
     *
     * @param int $optionIdx
     * @return array
     * @throws \CodeException
     * @throws \UserFormException
     */
    private function getFormElementForJsonCheckBox($htmlFormElementName, $value, array $formElement, $optionIdx = 0) {
        $addClassRequired = array();

        $json = HelperFormElement::getJsonFeMode($formElement[FE_MODE]); // disabled, required

        $json[API_FORM_UPDATE_FORM_ELEMENT] = $htmlFormElementName;

        if (isset($formElement[FE_FLAG_ROW_OPEN_TAG]) && isset($formElement[FE_FLAG_ROW_CLOSE_TAG])) {
            $flagRowUpdate = ($formElement[FE_FLAG_ROW_OPEN_TAG] && $formElement[FE_FLAG_ROW_CLOSE_TAG]);
        } else {
            $flagRowUpdate = true;
        }

        $statusHidden = ($formElement[FE_MODE] == 'hidden');

        // 'value' update via 'form-update' on the full row: only if there is no other FE in that row
        if ($flagRowUpdate) {
            $json[API_FORM_UPDATE_VALUE] = $value;
        }

        if ($formElement[FE_MODE] == FE_MODE_REQUIRED || $formElement[FE_MODE] == FE_MODE_SHOW_REQUIRED) {
            $addClassRequired = HelperFormElement::getRequiredPositionClass($formElement[F_FE_REQUIRED_POSITION]);
        }

        // Label
        if (isset($formElement[FE_LABEL])) {
            $key = $formElement[FE_HTML_ID] . HTML_ID_EXTENSION_LABEL;
            $json[API_ELEMENT_UPDATE][$key][API_ELEMENT_CONTENT] = HelperFormElement::buildLabel($htmlFormElementName, $formElement[FE_LABEL], $addClassRequired[FE_LABEL] ?? '');
        }

        // Note
        if (isset($formElement[FE_NOTE])) {
            $key = $formElement[FE_HTML_ID] . HTML_ID_EXTENSION_NOTE;
            if (!empty($addClassRequired[FE_NOTE])) {
                $formElement[FE_NOTE] = Support::wrapTag('<span class="' . $addClassRequired[FE_NOTE] . '">', $formElement[FE_NOTE]);
            }
            $json[API_ELEMENT_UPDATE][$key][API_ELEMENT_CONTENT] = $formElement[FE_NOTE];
        }

        // Input
        if (isset($formElement[FE_TYPE])) {
            $key = $formElement[FE_HTML_ID] . HTML_ID_EXTENSION_INPUT;

            // Check show/hide: only FE with FE_MODE_SQL given, might change.
            if (!empty($formElement[FE_MODE_SQL])) {
                $class = is_numeric($formElement[FE_BS_INPUT_COLUMNS]) ? ('col-md-' . $formElement[FE_BS_INPUT_COLUMNS]) : $formElement[FE_BS_INPUT_COLUMNS];

                $json[API_ELEMENT_UPDATE][$key][API_ELEMENT_ATTRIBUTE]['required'] = ($formElement[FE_MODE] == 'required');
                $json[API_ELEMENT_UPDATE][$key][API_ELEMENT_ATTRIBUTE]['hidden'] = $statusHidden;
                $json[API_ELEMENT_UPDATE][$key][API_ELEMENT_ATTRIBUTE]['readonly'] = $json['disabled'] ? 'readonly' : 'false';

                // Checkbox: Copy attributes to every checkbox (appreciated by screen reader):
                if ($formElement[FE_TYPE] == FE_TYPE_CHECKBOX) {
                    $optionsId = $formElement[FE_HTML_ID] . '-' . $optionIdx;
                    $json[API_ELEMENT_UPDATE][$optionsId][API_ELEMENT_ATTRIBUTE] = $json[API_ELEMENT_UPDATE][$key][API_ELEMENT_ATTRIBUTE];
                    $json[API_ELEMENT_UPDATE][$optionsId][API_ELEMENT_ATTRIBUTE]['data-disabled'] = $json['disabled'] ? 'yes' : 'no';
                    $json[API_ELEMENT_UPDATE][$optionsId][API_ELEMENT_ATTRIBUTE]['data-required'] = $json['required'] ? 'yes' : 'no';
                    $json[API_ELEMENT_UPDATE][$optionsId][API_ELEMENT_ATTRIBUTE]['disabled'] = $json['disabled'] ? 'disabled' : 'false';
                    $json[API_ELEMENT_UPDATE][$optionsId][API_ELEMENT_ATTRIBUTE]['required'] = $json['required'] ? 'required' : 'false';

                    // Update label class (i.e.: 'qfq-disabled') of Checkbox/Radio (i.e. readonly on/off).
                    if (isset($formElement[FE_TMP_CLASS_OPTION])) {
                        $optionsLabelId = HelperFormElement::getCheckboxRadioOptionId($formElement[FE_HTML_ID], $optionIdx, HTML_ID_EXTENSION_LABEL);
                        $json[API_ELEMENT_UPDATE][$optionsLabelId][API_ELEMENT_ATTRIBUTE]['class'] = $formElement[FE_TMP_CLASS_OPTION];
                    }
                }

                $class .= ($formElement[FE_MODE] == FE_MODE_HIDDEN) ? ' hidden' : '';
                $json[API_ELEMENT_UPDATE][$key][API_ELEMENT_ATTRIBUTE]['class'] = $class;

            }
        }

        // Show / Hide the complete FormElement Row.
        if (isset($formElement[FE_HTML_ID])) { // HIDDEN_SIP comes without a real existing FE structure.
            // Activate 'show' or 'hidden' on the current FormElement via JSON 'API_ELEMENT_UPDATE'
            $class = "form-group clearfix";
            if ($formElement[FE_MODE] == FE_MODE_HIDDEN) {
                $class .= ' hidden';
            }

            if (!empty($addClassRequired[FE_INPUT])) {
                $class .= ' ' . $addClassRequired[FE_INPUT];
            }

            $key = $formElement[FE_HTML_ID] . HTML_ID_EXTENSION_ROW;
            $json[API_ELEMENT_UPDATE][$key][API_ELEMENT_ATTRIBUTE]['class'] = $class;
//            $json[API_ELEMENT_UPDATE][$key][API_ELEMENT_ATTRIBUTE]['buggy'] = $class;
        }

        return $json;
    }

    /**
     * Build a single HTML plain checkbox based on two values.
     * Create a 'hidden' input field and a 'checkbox' input field - both with the same HTML 'name'.
     * HTML does not submit an unchecked checkbox. Then only the 'hidden' input field is submitted.
     *
     * <input name="$htmlFormElementName" type="radio" [autofocus="autofocus"]
     *            [required="required"] [disabled="disabled"] value="<value>" [checked="checked"] >
     *
     * @param array $formElement
     * @param string $htmlFormElementName
     * @param string $attribute
     * @param string $value
     * @param array $json
     *
     * @return string
     * @throws \CodeException
     * @throws \UserFormException
     */
    public function constructCheckboxSinglePlain(array $formElement, $htmlFormElementName, $attribute, $value, array &$json, $valueJson) {
        $html = '';

        $htmlHidden = HelperFormElement::buildNativeHidden($htmlFormElementName, $formElement[FE_CHECKBOX_UNCHECKED]);
        $this->store->setVar($htmlFormElementName, $htmlHidden, STORE_ADDITIONAL_FORM_ELEMENTS, false);
        $html = '';

        $html .= '<input ' . $attribute . '>';
        if (isset($formElement['label2'])) {
            $html .= Support::wrapTag("<span style='font-weight: 400;'>", $formElement['label2']);
        }

        $labelAttribute = Support::doAttribute('title', $formElement[FE_TOOLTIP]);

        $formElement[FE_BUTTON_CLASS] = 'checkbox';
        if ($formElement[FE_MODE] == FE_MODE_READONLY) {
            $formElement[FE_BUTTON_CLASS] .= ' qfq-disabled'; // necessary for own style checkboxes to display them 'disabled'
        }

        $formElement[FE_TMP_CLASS_OPTION] = $formElement[FE_BUTTON_CLASS];
        $labelAttribute = Support::doAttribute('title', $formElement[FE_TOOLTIP]);
        $labelAttribute .= Support::doAttribute('class', $formElement[FE_BUTTON_CLASS]);
        $labelAttribute .= Support::doAttribute('id', HelperFormElement::getCheckboxRadioOptionId($formElement[FE_HTML_ID], 0, HTML_ID_EXTENSION_LABEL));

        $html = Support::wrapTag("<label $labelAttribute>", $html, true);

        $json = $this->getFormElementForJsonCheckBox($htmlFormElementName, $valueJson, $formElement);

        return $html;
    }
}